export rho,
    Cp,
    visc,
    therm_cond,
    therm_expand,
    compress,
    std_props,
    cp0,
    cpP,
    CpPStd,
    CpPVar,
    cpF,
    kyaws,
    kfit,
    enthalpy

struct CpPStd end
struct CpPVar end
struct CpFit end

const cp0 = CpPStd()
const cpP = CpPVar()
const cpF = CpFit()


struct kYaws end
struct kFit end

const kyaws = kYaws()
const kfit = kFit()


## Std props
function std_props(fluid::Fluid)
    ρ = fluid.ρ
end


## rho

function rho(P, T, fluid::Fluid)
    @error "Property not yet implemented for the fluid. - ρ"
    throw(NotImplementedError())
    return missing
end


## Cp

function Cp(P, T, fluid::Fluid)
    @error "Property not yet implemented for the fluid. - Cₚ"
    throw(NotImplementedError())
    return missing
end


## k
function therm_cond(P, T, fluid::Fluid)
    @error "Property not yet implemented for the fluid. - k"
    throw(NotImplementedError())
    return missing
end


# mu
function visc(P, T, fluid::Fluid)
    @error "Property not yet implemented for the fluid. - μ"
    throw(NotImplementedError())
    return missing
end

# Solid

## density

function rho(P, T, solid::Solid)
    # @error "Property not yet implemented for solid - ρ"
    # throw(NotImplementedError())
    # return missing
    return solid.ρ
end

## Cp

function Cp(P, T, solid::Solid)
    # @error "Property not yet implemented for solid - Cₚ"
    # throw(NotImplementedError())
    # return missing
    return solid.cₚ
end


# function Cp(P, T, solid::CaCO3)
#     return solid.cₚ
# end

# function Cp(P,T,solid::Hematite)
#     A=93.43834
#     B=108.3577
#     C=-50.86447
#     D=25.58683
#     E=-1.611330
#     t=T/1000.0
#     CpM=A+B*t+C*t^2+D*t^3+E/(t^2)
#     return CpM/159.688*1000.0
# end

## therm cond

function therm_cond(P, T, solid::Solid)
    @error "Property not yet implemented for solid - k"
    throw(NotImplementedError())
    return missing
end






function drho(P,T,mat::Material)
    return gradient((PT)->rho(PT[1],PT[2],mat),[P,T])
end

drhodP(P,T,mat::Material) = drho(P,T,mat)[1]
drhodT(P,T,mat::Material) = drho(P,T,mat)[2]


function therm_expand(P,T,mat::Material)
    return -drhodT(P,T,mat)/rho(P,T,mat)
end

function compress(P,T,mat::Material)
    return drhodP(P,T,mat)/rho(P,T,mat)
end


function dbeta(P,T,mat::Material)
    return gradient((PT)->therm_expand(PT[1],PT[2],mat),[P,T])
end


dbetadT(P,T,mat::Material) = dbeta(P,T,mat)[2]



function Cp(P,T,fluid::Fluid,model::CpPVar)
    Cp0=Cp(P,T,fluid,cp0)
    f(Ps,args) = (therm_expand(Ps,T,fluid)^2+dbetadT(Ps,T,fluid))/rho(Ps,T,fluid)

    prob=IntegralProblem(f,101325.0,P,[])
    sol=solve(prob,QuadGKJL())#,reltol=1E-12,abstol=1E-12)#,nout=1,batch=0)

    return Cp0-T*sol[1]
end


struct enthalpFast end
struct enthalpSlow end

function enthalpy(P,T,fluid::Fluid,model::enthalpSlow=enthalpSlow())

    P_std=101325.0
    T_std=273.15+20

    volume_term(Ps,args) = (1.0+therm_expand(Ps,T,fluid)*T)/rho(Ps,T,fluid)
    energy_term(Ts,args) = Cp(P_std,Ts,fluid)

    prob1=IntegralProblem(volume_term,P_std,P,[])
    prob2=IntegralProblem(energy_term,T_std,T,[])

    sol1=solve(prob1,QuadGKJL(),reltol=1E-15,abstol=1E-12)
    sol2=solve(prob2,QuadGKJL(),reltol=1E-15,abstol=1E-12)

    return sol1[1]+sol2[1]
end
