export generate_JLD2,
        generate_XLSX



function generate_JLD2(fname,props_dics,Ts,Ps,fluid;include_std=true)

    pair_PT_table=[(P=P,T=T) for P in Ps, T in Ts]
    pair_PT=[pair_PT_table...]

    len_PT=length(pair_PT)
    jump_T=length(Ts)
    size_PT=size(pair_PT_table)

    props_keys=keys(props_dics)



    props_lines=Dict{Union{Symbol,String},Array{Float64}}(
        [k => zeros(Float64,len_PT) for k in props_keys]
    )

    props_tabs=Dict{Union{Symbol,String},Array{Float64}}(
        [k => zeros(Float64,size_PT) for k in props_keys]
    )



    for k in props_keys
        for (i, P) in enumerate(Ps)
            for (j, T) in enumerate(Ts)
                prop_calc = props_dics[k](P, T, fluid)

                props_tabs[k][i, j] = prop_calc

                props_lines[k][(i-1)*jump_T+j] = prop_calc

            end
        end
    end




    jldopen(fname,"w") do file
        file["fluid"]=fluid

        file["inputs/Ts"]=Ts
        file["inputs/Ps"]=Ps
        file["inputs/pairs"]=pair_PT
        file["inputs/table"]=pair_PT_table

        file["inputs/len_data"]=len_PT
        file["props"]=[k for k in props_keys]

        file["data/linear"]=props_lines
        file["data/tabular"]=props_tabs


    end


    return true

end


function generate_XLSX(fname,props_dics,Ts,Ps,fluid::Union{Fluid,DrillFluid},units;T_unit="K",P_unit="Pa")

    println((P_unit=P_unit,T_unit=T_unit))
    if T_unit == "K"
        Ts_SI=Ts
    elseif T_unit == "°C"
        Ts_SI=Ts.+273.15
    else
        error("Unit conversion not implemented! Check if unit string is correct.")
    end
    if P_unit == "Pa"
        Ps_SI=Ps
    elseif P_unit == "bar"
        Ps_SI = Ps.*1.0E5
    else
        error("Unit conversion not implemented! Check if unit string is correct.")
    end

    pair_PT_table=[(P=P,T=T) for P in Ps_SI, T in Ts_SI]
    pair_PT=[pair_PT_table...]

    size_PT=size(pair_PT_table)
    props_keys=keys(props_dics)
    props_tabs=Dict{Union{Symbol,String},Array{Float64}}(
        [k => zeros(Float64,size_PT) for k in props_keys]
    )


    for k in props_keys
        for (i, P) in enumerate(Ps_SI)
            for (j, T) in enumerate(Ts_SI)
                prop_calc = props_dics[k](P, T, fluid)
                props_tabs[k][i, j] = prop_calc
            end
        end
    end


    openxlsx(fname,mode="w") do xf

        for (i,k) in enumerate(props_keys)
            if i ==1
                sh=xf[1]
                rename!(sh,k)
            else
                addsheet!(xf,k)
                sh=xf[i]
            end
            # rename!(sh,k)
            sh["A1"] = "Unit:"
            sh["B1"] = "["*units[k]*"]"
            sh["C2"] = "T ["*T_unit*"]"
            sh["B3"] = "P ["*P_unit*"]"
            # writetable!(sh,Ts',[_ for T in Ts],anchor_cell=CellRef("D2"))
            # writetable!()
            sh["D2", dim = 2] = Ts
            sh["B4", dim = 1] = Ps

            sh["D4"] = props_tabs[k]
        end
    end

end
